class File:
  def __init__(self, name, parent):
    self.parent = parent
    self.name = name
    if parent != None and self.parent.isDir() == True:
      self.parent.addChild(self)
      
  def getName(self):
    return self.name
    
  def isDir(self):
    return False
    
class Dir(File):
  def __init__(self, name, parent):
    # prima di invocare il costruttore della classe base!
    # Altrimenti ripulisco la lista dei figli dopo averla
    # inizializzata nel costruttore di File
    #
    self.children = []
    # costruttore classe base
    File.__init__(self, name, parent)
    
  def addChild(self, file):
  #  print('adding child with name ' + file.getName() + ' to ' +  self.getName())
    self.children.append(file)
    
  def getChildren(self):
    return self.children
    
  def isDir(self):
    return True
    

class FileManager:
  def __init__(self, directory):
    # wdir: working directory
    self.wdir = directory
    
  
  def pwd(self):
    """Prints working directory"""
    print(self.path(self.wdir))
    
  def ls(self):
    """Lists the contents of current directory"""
    for f in self.wdir.children:
      print(f.getName())
      
  def cd(self, name):
    """Sets the current working directory to the given one, if exists.
       If name is "..", then changes to the parent directory
    """
    
    if name == ".." and self.wdir.parent is not None:
      self.wdir = self.wdir.parent
    else:
    
      f = self.find(name)
      if f is not None and f.isDir() == True:
        self.wdir = f
      elif f is not None and not f.isDir():
        print("Error: cannot change dir to " + f.getName() + ": it is not a valid directory")
      else:
        print("Directory " + name + " not found under " + self.wdir.getName())
      
  def path(self, file):
     s = file.getName()
     parent = file.parent
     
     while parent != None:
       parentName = parent.getName()
       if parentName != '/':
         s = parentName + '/' + s
       else:
         s = parentName + s
 
       
       parent = parent.parent
       
     return s
     
  def recursive_find(self, file, name):
   #  print("Searching inside " + file.getName())
     
     if file.getName() == name:
       return file
       
     if file.isDir():
       for f in file.children:
         found = self.recursive_find(f, name)
         if found is not None:
            return found

     return None
     
     
  def find(self, name):
     """Returns a file (or directory) with the provided name, if it exists. 
        Returns None otherwise.
        Note: if more than one file in the system has the same name, then the first
        file which is found is returned.
     """
#     print("Find inside " + self.wdir.getName())
     
     return self.recursive_find(self.wdir, name)
     
  def print_children(self, curfile, level):
     """recursion here!"""
     l = 0
     # indent 
     for i in range(0, level):
       print("  ", end="")
     
     # print a + near a directory, a - near a simple file

     if curfile.isDir():
       print("+ " + curfile.getName())
       for f in curfile.children:
         self.print_children(f, level + 1) # call print_children recursively
     else:
       print("- " + curfile.getName())   

         
   
  def tree(self):
    """Prints the tree of directories and files in the file system """
    self.print_children(self.wdir, 0)
     
     
     
     
     
       
       
       
       
       
       
       
       
       
       
       
       
       
       
       
       
     
      
  
    
    
    